/**************************************************************************
** File: xreg1.c
**
** Desc: Example of RegisterForEvent
**
**       Example of RegisterForEvent on EVENT_MODULE_LOAD and
**       EVENT_MODULE_UNLOAD.  This example uses a Local Semaphore
**       and a circular queue.  It also is multi-threaded in nature.
**
**
**    QMK386 XREG1 /z
**
**
**   DISCLAIMER  
**  
**   Novell, Inc. makes no representations or warranties with respect to
**   any NetWare software, and specifically disclaims any express or
**   implied warranties of merchantability, title, or fitness for a
**   particular purpose.  
**
**   Distribution of any NetWare software is forbidden without the
**   express written consent of Novell, Inc.  Further, Novell reserves
**   the right to discontinue distribution of any NetWare software.
**   
**   Novell is not responsible for lost profits or revenue, loss of use
**   of the software, loss of data, costs of re-creating lost data, the
**   cost of any substitute equipment or program, or claims by any party
**   other than you.  Novell strongly recommends a backup be made before
**   any software is installed.   Technical support for this software
**   may be provided at the discretion of Novell.
**
**
** Programmers:
**
**    Ini   Who                  Firm
**    ---------------------------------------------------------------------
**    DWH   Dirk W. Howard       Novell Developer Support
**
**
** History:
**
**    When        Who      What
**    ---------------------------------------------------------------------
**    10-5-1994    DWH      First code.
**
*/

/****************************************************************************
** Include headers, macros, function prototypes, etc.
*/

   /*------------------------------------------------------------------------
   ** ANSI
   */
   #include <stdio.h>      /* time(), ctime(), printf(), gets()         */
   #include <string.h>     /* strupr()                                  */
   #include <errno.h>      /* Ansi errno codes                          */

   /*------------------------------------------------------------------------
   ** NetWare
   */
   #include <advanced.h>   /* RegisterForEvent(), UnregisterForEvent()  */
   #include <conio.h>      /* ConsolePrintf(), kbhit(), getch()         */
   #include <process.h>    /* GetThreadGroupID(), SetThreadGroupID(), 
                              ReturnNLMVersionInformation(), exit(), 
                              ThreadSwitchWithDelay(), ExitThread(), 
                              BeginThread()                             */
   #include <signal.h>     /* signal()                                  */
   #include <time.h>       /* time(), ctime()                           */
   #include <fcntl.h>      /* sopen()                                   */
   #include <io.h>         /* sopen()                                   */
   #include <share.h>      /* sopen()                                   */
   #include <sys\stat.h>   /* sopen()                                   */
   #include <niterror.h>   /* NetWare error codes                       */
   #include <nwsemaph.h>   /* OpenLocalSemaphore(), CloseLocalSemaphore(),
                              WaitOnLocalSemaphore(), 
                              SignalLocalSemaphore()                    */

   /*------------------------------------------------
   ** Macros
   */
   #define ERROR_LOG    "SYS:SYSTEM\\SYS$LOG.ERR"
   #define MAX_QUEUE    5

/****************************************************************************
** Global variables
*/
struct ldEventQueueElement
{
   time_t      timeOfEvent;
   char        action[10];
   char        NLMdesc[250];
};

struct ldEventQueueElement ldEventQueue[MAX_QUEUE];
BYTE  qStart = 0;
BYTE  qEnd   = 0;

LONG  queueEventSemaphore = 0;

LONG  loadEventHandle = 0,
      unloadEventHandle = 0;
int   exitingNLM    = FALSE;
int   theTGID;

/****************************************************************************
** UnloadProcedure
**    insure that the NLM shuts down cleanly
*/
void UnloadProcedure()
{
   if ( loadEventHandle != 0 )
      if ( UnregisterForEvent( loadEventHandle ) != NULL )
         ConsolePrintf( "\n\rXREG1.UP: Failure to unregister "
                        "MODULE_LOAD event!\n\r" );

   if ( unloadEventHandle != 0 )
      if ( UnregisterForEvent( unloadEventHandle ) != NULL )
         ConsolePrintf( "\n\rXREG1.UP: Failure to unregister "
                        "MODULE_UNLOAD event!\n\r" );

   if ( queueEventSemaphore != 0 )
      CloseLocalSemaphore( queueEventSemaphore );
}


/**************************************************************************
** EventQueueProcedure
*/
void EventQueueProcedure()
{
   struct tm   *rawTime;
   int         hour;
   int         handle;
   char        ampm[4];
   char        timeText[40];
   char        tempBuf[250];

   while ( !exitingNLM )
      {
      WaitOnLocalSemaphore( queueEventSemaphore );

      /*------------------------------------------------
      ** Check to see if we are exiting
      */
      if ( exitingNLM )
         ExitThread( EXIT_THREAD, 0 );

      /*------------------------------------------------
      ** Check to see if there is an event to process.
      */
      if ( qStart == qEnd )
         {
         ConsolePrintf( "XREG1.EQP: No event found on queue.\n\r" );
         continue;
         }

      /*------------------------------------------------
      ** Convert event time to a string
      */
      rawTime = localtime( &ldEventQueue[qEnd].timeOfEvent );
      if ( rawTime->tm_hour > 11 )
         {
         if ( rawTime->tm_hour > 12 )
            hour = rawTime->tm_hour - 12;
         else
            hour = rawTime->tm_hour;
         strcpy( ampm, "pm" );
         }
      else
         {
         if ( rawTime->tm_hour == 0 )
            hour = 12;
         else
            hour = rawTime->tm_hour;
         strcpy( ampm, "am" );
         }
      sprintf( timeText, "%02d-%02d-%02d  %2d:%02d:%02d %s", 
               rawTime->tm_mon+1, rawTime->tm_mday, rawTime->tm_year, 
               hour, rawTime->tm_min, rawTime->tm_sec, ampm );

      /*------------------------------------------------
      ** Open system error log file in shared access
      */
      handle = sopen( ERROR_LOG, O_RDWR | O_APPEND | O_CREAT, SH_DENYNO, 0 );

      sprintf( tempBuf,  "%s:    XREG1\n   \"%s\" NLM %s\n\n", timeText, 
         ldEventQueue[qEnd].NLMdesc, ldEventQueue[qEnd].action );
      printf( "%s", tempBuf );

      if ( handle == -1 )
         {
         perror( "XREG1: sopen" );
         printf( "XREG1: NWerror %d\r\n", NetWareErrno );
         }
      else
         {
         if ( write( handle, tempBuf, strlen( tempBuf ) ) == -1 )
            {
            perror( "XREG1: write" );
            printf( "XREG1: NWerror %d\r\n", NetWareErrno );
            }
         close( handle );
         }
      ++qEnd;
      if ( qEnd >= MAX_QUEUE )
         qEnd = 0;
      }
}

/****************************************************************************
** LoadEventProcedure
**    procedure that is called when an EVENT_MODULE_LOAD is detected.
**    This event may sleep.
*/
void LoadEventProcedure( LONG moduleHandle )
{
   int         TGID;
   LONG        majVer;
   LONG        minVer;
   LONG        rev;
   LONG        year;
   LONG        month;
   LONG        day;
   BYTE        tempBuf[250];

   TGID = SetThreadGroupID( theTGID );

   ReturnNLMVersionInformation(
      /* int NLMHandle         */  moduleHandle,
      /* LONG *majorVersion    */  &majVer,
      /* LONG *minorVersion    */  &minVer,
      /* LONG *revision        */  &rev,
      /* LONG *year            */  &year,
      /* LONG *month           */  &month,
      /* LONG *day             */  &day,
      /* BYTE *copyrightString */  tempBuf,
      /* BYTE *description     */  ldEventQueue[qStart].NLMdesc
      );

   strcpy( ldEventQueue[qStart].action, "loaded" );
   ldEventQueue[qStart].timeOfEvent = time( NULL );
   ++qStart;
   if ( qStart >= MAX_QUEUE )
      qStart = 0;

   SignalLocalSemaphore( queueEventSemaphore );

   SetThreadGroupID( TGID );
}


/****************************************************************************
** UnloadEventProcedure
**    procedure that is called when an EVENT_MODULE_UNLOAD is detected.
**    This event may sleep.
*/
void UnloadEventProcedure( LONG moduleHandle )
{
   int         TGID;
   LONG        majVer;
   LONG        minVer;
   LONG        rev;
   LONG        year;
   LONG        month;
   LONG        day;
   BYTE        tempBuf[250];

   TGID = SetThreadGroupID( theTGID );

   ReturnNLMVersionInformation(
      /* int NLMHandle         */  moduleHandle,
      /* LONG *majorVersion    */  &majVer,
      /* LONG *minorVersion    */  &minVer,
      /* LONG *revision        */  &rev,
      /* LONG *year            */  &year,
      /* LONG *month           */  &month,
      /* LONG *day             */  &day,
      /* BYTE *copyrightString */  tempBuf,
      /* BYTE *description     */  ldEventQueue[qStart].NLMdesc
      );

   strcpy( ldEventQueue[qStart].action, "unloaded" );
   ldEventQueue[qStart].timeOfEvent = time( NULL );
   ++qStart;
   if ( qStart >= MAX_QUEUE )
      qStart = 0;

   SignalLocalSemaphore( queueEventSemaphore );

   SetThreadGroupID( TGID );
}


/****************************************************************************
**    main
**        main body of the program
*/
void main()
{
   char  szTemp[256];

   theTGID = GetThreadGroupID();

   /*----------------------------------------------------------------------
   ** pass UnloadProcedure to the signal handler
   */
   signal(
      /* int signal   */    SIGTERM,
      /* void (*func) */    UnloadProcedure
      );

   /*--------------------------------------------------------------------
   ** Register for notification on the events EVENT_MODULE_LOAD and
   ** EVENT_MODULE_UNLOAD
   */
   loadEventHandle = RegisterForEvent(
                        /* LONG eventType         */    EVENT_MODULE_LOAD,
                        /* void (*reportProcedure */    LoadEventProcedure,
                        /* LONG (*warnProcedure   */    NULL
                        );
    if ( loadEventHandle == 0 )
       {
       ConsolePrintf( "\n\rXREG1: Failure to register for event "
                      "EVENT_MODULE_LOAD.\n\r" );
       exit( 1 );
       }

    unloadEventHandle = RegisterForEvent(
                      /* LONG eventType         */    EVENT_MODULE_UNLOAD,
                      /* void (*reportProcedure */    UnloadEventProcedure,
                      /* LONG (*warnProcedure */      NULL
                      );
   if ( unloadEventHandle == 0 )
       {
       ConsolePrintf( "\n\rXREG1: Failure to register for event "
                      "EVENT_MODULE_UNLOAD.\n\r" );
       exit( 1 );
      }

   queueEventSemaphore = OpenLocalSemaphore( 0 );
   if ( queueEventSemaphore == 0 )
      {
      ConsolePrintf( "XREG1.MAIN: Error opening local semaphore handle" );
      return;
      }
   BeginThread( EventQueueProcedure, NULL, NULL, NULL );

   while ( !exitingNLM )
      {
      ThreadSwitchWithDelay();
      if ( kbhit() )
         {
         /*------------------------------------------------------------------
         ** Flush the keyboard buffer.
         */
         while( kbhit() ) 
            if ( !getch() ) 
               getch();

         do
            {
            printf( "XREG1: Exit? (Y/N): " );
            gets( szTemp );
            strupr( szTemp );
            } while( (szTemp[0] != 'N') && (szTemp[0] != 'Y') );

         if ( szTemp[0] == 'Y' )
            {
            exitingNLM=TRUE;
            break;
            }
         }
      }

   SignalLocalSemaphore( queueEventSemaphore );

   if ( loadEventHandle != 0 )
      if ( UnregisterForEvent( loadEventHandle ) != NULL )
         ConsolePrintf( "\n\rXREG1.UP: Failure to unregister "
                        "MODULE_LOAD event!\n\r" );
   if ( unloadEventHandle != 0 )
       if ( UnregisterForEvent( unloadEventHandle ) != NULL )
          ConsolePrintf( "\n\rXREG1.UP: Failure to unregister "
                         "MODULE_UNLOAD event!\n\r" );

   if ( queueEventSemaphore != 0 )
      CloseLocalSemaphore( queueEventSemaphore );

   ExitThread(
      /* int actionCode */    EXIT_THREAD,
      /* int status     */    0
      );
}
